/* GROOVE: GRaphs for Object Oriented VErification
 * Copyright 2003--2023 University of Twente
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * $Id$
 */
package nl.utwente.groove.grammar.rule;

import java.util.Collections;
import java.util.List;
import java.util.Set;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

import nl.utwente.groove.grammar.type.TypeElement;
import nl.utwente.groove.grammar.type.TypeGuard;
import nl.utwente.groove.graph.Element;

/**
 * Type of (node and edge) elements that may appear in a {@link RuleGraph}.
 * Super-interface of {@link RuleNode}s and {@link RuleEdge}s.
 * @author Arend Rensink
 * @version $Revision$
 */
@NonNullByDefault
public interface RuleElement extends Element, AnchorKey {
    /** Returns the type of this rule element. */
    public @Nullable TypeElement getType();

    /** Returns the set of label variables associated with this rule element. */
    default public Set<LabelVar> getVars() {
        return EMPTY_VAR_SET;
    }

    /** Returns the collection of (named) type guards associated with this rule element. */
    default public List<TypeGuard> getTypeGuards() {
        return EMPTY_GUARD_LIST;
    }

    /**
     * Returns the set of type elements that are valid matches of this rule element.
     * This is typically the set of subtypes of {@link #getType()}, but it may be
     * further constrained by type variables.
     * @see #getType()
     */
    public Set<? extends TypeElement> getMatchingTypes();

    /** Fixed global empty set of label variables. */
    final static Set<LabelVar> EMPTY_VAR_SET = Collections.emptySet();
    /** Fixed global empty set of label variables. */
    final static List<TypeGuard> EMPTY_GUARD_LIST = Collections.emptyList();
}
