/* GROOVE: GRaphs for Object Oriented VErification
 * Copyright 2003--2023 University of Twente
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * $Id$
 */
package nl.utwente.groove.util.parse;

import org.eclipse.jdt.annotation.NonNullByDefault;

/**
 * Checker interface that tests for format errors in objects of a given type.
 * @param <T> type of the objects to be checked
 * @author Arend Rensink
 * @version $Revision$
 */
@NonNullByDefault
public interface FormatChecker<T> {
    /**
     * Checks the errors in a given value.
     * @return the (possibly empty) set of errors in the value
     */
    public FormatErrorSet check(T value);

    /** Default checker that always return the empty error set. */
    public static final FormatChecker<Object> EMPTY_CHECKER = new FormatChecker<>() {
        @Override
        public FormatErrorSet check(Object value) {
            return new FormatErrorSet();
        }
    };

    /** Default checker that always return the empty error set. */
    public static final FormatChecker<String> EMPTY_STRING_CHECKER = new FormatChecker<>() {
        @Override
        public FormatErrorSet check(String value) {
            return new FormatErrorSet();
        }
    };
}
